# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase
from django.test.client import RequestFactory
from django.conf import settings
from django.contrib.auth.models import AnonymousUser

from ..authentication import KongOAuthAuthentication
from ..helpers import get_redis
from ..user import TransientUser

from .testutils import get_user_data

import json

class AuthenticationSuccessTestCase(TestCase):

    def setUp(self):
        self.rf = RequestFactory()
        self.auth = KongOAuthAuthentication()
        self.user_id = 1

        headers = {
            'HTTP_X_AUTHENTICATED_USERID': self.user_id
        }
        request = self.rf.get('/', **headers)
        self.user, self.result = self.auth.authenticate(request)

    def test_successfully_logs_in(self):
        assert self.result is None

    def test_autenticates_user(self):
        assert self.user.id == self.user_id

    def test_returns_transient_user(self):
        assert isinstance(self.user, TransientUser)


class AuthenticationFailureTestCase(TestCase):

    def setUp(self):
        self.rf = RequestFactory()
        self.auth = KongOAuthAuthentication()
        self.user_id = 1

        headers = {
            'HTTP_X_ANONYMOUS_CONSUMER': 'anon'
        }
        request = self.rf.get('/', **headers)
        self.user, self.result = self.auth.authenticate(request)

    def test_it_returns_an_anon_user(self):
        assert isinstance(self.user, AnonymousUser)

    def test_it_returns_logged_in_false(self):
        assert self.result == False


class AuthenticationUserSerializationTestCase(TestCase):

    def setUp(self):
        redis = get_redis()
        self.user_id = 1
        user_permission_data = get_user_data(self.user_id)
        redis.set('authorization.1', user_permission_data)
        self.auth = KongOAuthAuthentication()

    def test_it_gets_user_from_redis(self):
        result = self.auth.get_user(self.user_id)






from rest_framework import permissions


class KongOAuthPermission(permissions.BasePermission):
    """
    Determines if user has the correct permissions for a class based view,

    add to class based view (Not that Function Based Views can use the
    permission_required decorator with KongOAuthPermission):

    permission_classes = (KongOAuthPermission,)
    permission_list = ['group_1', 'group_2]

    this will then check a user and see if they have the required permissions
    """

    def has_permission(self, request, view):
        method_mapper = {
            'get': 'list',
            'post': 'create',
            'put': 'update',
            'patch': 'update',
            'delete': 'destroy'
        }
        return self.check_permissions(
            request, view.permission_list,
            action=method_mapper[request.method.lower()]
        )

    @staticmethod
    def check_permissions(request, permission_list, **kwargs):
        if kwargs['action'] in permission_list:
            for permission in permission_list[kwargs['action']]:
                if request.user.has_perm(permission):
                    return True
            return False
        return True



from rest_framework.test import APIClient as Client
from .testutils import get_user_data
from django.test import TestCase
from ..helpers import get_redis
from example.api import Example


class KongOAuthPermissionTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        redis = get_redis()
        self.user_id = 1
        self.example = Example.objects.create(name='example')
        user_permission_data = get_user_data(self.user_id)
        redis.set('authorization.1', user_permission_data)
        self.headers = {
            'HTTP_X_AUTHENTICATED_USERID': self.user_id
        }

    def test_get_request_has_permissions(self):
        response = self.client.get('/example/', **self.headers)
        self.assertTrue(response.status_code == 200,
                        msg='Valid Permissions Fail')

    def test_patch_request_has_no_permissions(self):
        response = self.client.patch(
            '/example/{}/'.format(self.example.id),
            **self.headers
        )
        self.assertTrue(response.status_code == 200,
                        msg='Valid Permissions Fail')

    def test_put_request_has_no_permissions(self):
        response = self.client.put(
            '/example/{}/'.format(self.example.id),
            data={
                'name': 'example'
            },
            **self.headers
        )
        self.assertTrue(response.status_code == 200,
                        msg='Valid Permissions Fail')

    def test_post_has_invalid_permissions(self):
        response = self.client.post(
            '/example/',
            data={
                'name': 'example'
            },
            **self.headers
        )
        self.assertTrue(response.status_code == 403,
                        msg='Invalid Permissions Passes')



from django.conf import settings
import redis


def get_redis():
    """
    Get a redis connection
    # rediss://:password@hostname:port/0
    """
    defualt_connection = {
        "host": "redis"
    }
    defualt_connection.update(getattr(settings, 'REDIS_CONN', {}))
    # return redis.Redis(**{"host": "redis"})
    return redis.Redis(**defualt_connection)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.test import TestCase
from .testutils import get_user_data

from ..user import TransientUser

import json


class UserTestCase(TestCase):

    def setUp(self):
        self.user_id = 1
        data = json.loads(get_user_data(self.user_id))
        self.user = TransientUser.from_redis_data(data)

    def test_it_sets_user_id(self):
        assert self.user.id == self.user_id

    def test_it_sets_groups(self):
        assert len(self.user.groups) == 1

    def test_it_sets_user_permissions(self):
        assert len(self.user.user_permissions) == 4

    def test_it_can_get_permissions_by_group(self):
        pass

    def test_it_can_verify_user_has_permission(self):
        assert self.user.has_perm('all') == True,\
            'Expected the user to have the all permission'

    def test_it_can_verify_user_does_not_have_permission(self):
        assert self.user.has_perm('illegal-permission') == False,\
            'Expected the user not to have the foo permission'

    def test_has_permissions(self):

        tests = [
            # input, expected result
            (['foo','bar','baz'], True), # user has more permissions than list provided
            (['not a permission','bar','baz'], False), # user has some permission, but is missing some
        ]
        for test_input, expected_result in tests:
            assert self.user.has_perms(test_input) == expected_result,\
                'Expected: {} to be: {}'.format(test_input, expected_result)



from django.contrib.auth.models import AnonymousUser
from rest_framework import authentication
from django.conf import settings
from .user import TransientUser
from .helpers import get_redis
import json


class KongOAuthAuthentication(authentication.BaseAuthentication):

    def authenticate(self, request):
        """
        Uses Kong's OAuth headers to create a a 'Transit' User
        note: this may need to be X_ (not HTTP_X_..) in real life:
        """
        anonymous = request.META.get("HTTP_X_ANONYMOUS_CONSUMER", None)
        if not anonymous:
            user_id = request.META.get("HTTP_X_AUTHENTICATED_USERID", None)
            user = self.get_user(user_id)
            return user, None
        else:
            return AnonymousUser(), False

    @staticmethod
    def get_user(user_id):
        """
        Gets User information from a permission redis instance
        """
        redis_key = getattr(settings, 'AUTH_REDIS_KEY',
                            'authorization.{}').format(user_id)
        redis = get_redis()
        user_data = redis.get(redis_key)
        user_data = json.loads(user_data)
        return TransientUser.from_redis_data(user_data)


class KongKeyAuthAuthentication(authentication.BaseAuthentication):

    def authenticate(self, request):
        pass



from kongoauth.authentication import KongOAuthAuthentication
from kongoauth.permissions import KongOAuthPermission
from rest_framework import routers
from django.db import models
from rest_framework import (
    serializers,
    viewsets,
)


class Example(models.Model):
    name = models.CharField(max_length=10)


class ExampleSerializer(serializers.ModelSerializer):

    class Meta:
        model = Example
        fields = '__all__'


class ExampleViewSet(viewsets.ModelViewSet):
    authentication_classes = (KongOAuthAuthentication,)
    permission_classes = (KongOAuthPermission,)
    serializer_class = ExampleSerializer
    queryset = Example.objects.all()
    permission_list = {
        'list': ['all'],
        'create': ['protected']
    }


router = routers.DefaultRouter()
router.register(
    prefix=r'example',
    viewset=ExampleViewSet,
    base_name='example'
)




from django.contrib.auth import get_user_model

class Group:
    name = None
    permissions = []

    def __init__(self, name, permissions):
        self.name = name
        self.permissions = permissions

class Permission:
    name = None
    code = None

    def __init__(self, name, code):
        self.name = name
        self.code = code


class TransientUser:
    '''
    A simple User class that mimics the standard Django user
    '''
    id = None
    groups = []
    user_permissions = []

    def __init__(self, id, groups, permissions):
        self.id = id
        self.groups = groups
        self.user_permissions = permissions

    @classmethod
    def from_redis_data(cls, data):
        user_id = data.get('id', None)
        orgs = data.get('organizations', [])
        groups = []

        user_permissions = set([])
        for org in orgs:
            groups = org.get('groups', [])
            for group in groups:
                group_name = group.get('name')
                permissions = group.get('permissions', [])
                grp = Group(group_name, permissions)

                permission_codes = [permission.get('code')
                                    for permission in permissions]
                user_permissions.update(permission_codes)

        user = cls(user_id, groups, user_permissions)
        return user

    def get_group_permissions(self, obj=None):
        '''
        Returns a set of permission strings that the user has, through their groups.

        If obj is passed in, only returns the group permissions for this specific object.
        '''
        pass

    def get_all_permissions(self, obj=None):
        '''
        Returns a set of permission strings that the user has, both through group and user permissions.
        If obj is passed in, only returns the permissions for this specific object.
        '''


    def has_perm(self, perm, obj=None):
        """
        Returns True if the user has the specified permission, where perm is in the format "<app label>.<permission codename>". (see documentation on permissions). If the user is inactive, this method will always return False.

        If obj is passed in, this method won’t check for a permission for the model, but for this specific object.
        """
        return perm in self.user_permissions

    def has_perms(self, perm_list, obj=None):
        '''
        Returns True if the user has each of the specified permissions, where each perm is in the format "<app label>.<permission codename>". If the user is inactive, this method will always return False.

        If obj is passed in, this method won’t check for permissions for the model, but for the specific object.
        '''
        required_permissions = set(perm_list)
        union = required_permissions.intersection(self.user_permissions)
        return len(union) == len(required_permissions)





import json


def get_logged_in_headers():
    pass


def get_anon_headers(self):
    pass


def get_user_data(user_id, orgs = None, groups = None):
    """Get an example redis user object"""

    if groups is None:
        groups = [
            {
                'name': 'organization_admin',
                'permissions': [
                    {'name': 'all', 'code': 'all'},
                    {'name': 'foo', 'code': 'foo'},
                    {'name': 'bar', 'code': 'bar'},
                    {'name': 'baz', 'code': 'baz'}
                ]
            }
        ]

    if orgs is None:
        orgs = [
            {
                'name': 'organization.admin',
                'id': 1,
                'groups': groups
            }
        ]

    return json.dumps({
        'id': user_id,
        'organizations': orgs
    })
